"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = __importStar(require("http"));
class HttpServer {
    constructor(options) {
        this.destroyed = false;
        this.connections = new Map();
        this.tag = options.tag || 'http-server';
        this.log = options.log.child({ tag: this.tag });
        const server = options.server || http.createServer();
        server.on('request', options.handler);
        this.server = server;
        this.listenOptions = options.listen;
    }
    trackConnection(socket) {
        const conn = { socket, idle: true };
        this.connections.set(socket, conn);
        socket.on('close', () => {
            this.connections.delete(socket);
        });
    }
    trackRequest(req, res) {
        const conn = this.connections.get(req.socket);
        conn.idle = false;
        res.on('finish', () => {
            conn.idle = true;
            this.destroyConnection(conn);
        });
    }
    destroyConnection(conn) {
        if (this.destroyed && conn.idle)
            conn.socket.destroy();
    }
    async init() {
        const server = this.server;
        await new Promise((resolve, reject) => {
            server.once('error', reject);
            server.listen(this.listenOptions, () => {
                server.removeListener('error', reject);
                server.on('connection', socket => this.trackConnection(socket));
                server.on('request', (req, res) => {
                    this.trackRequest(req, res);
                });
                const address = server.address();
                this.log.info({ address }, 'listening');
                resolve();
            });
        });
    }
    async destroy() {
        await new Promise(resolve => {
            this.server.once('close', resolve);
            this.server.close();
            this.destroyed = true;
            for (const conn of this.connections.values())
                this.destroyConnection(conn);
        });
        this.log.info('closed');
    }
}
exports.default = HttpServer;
//# sourceMappingURL=index.js.map